pub struct FrameEntryOwner {
pub mapped_pa: usize,
pub size: usize,
pub prop: PageProperty,
pub is_tracked: bool,
}Expand description
Verification Design
The owner type for a page table leaf: a frame mapped in a node. Asterinas supports huge pages, so it is not necessarily at level 1.
mapped_pais the physical address of the mapped frame.sizeis the size of the mapping, which corresponds to the level of the parent page table. 4k at level 1, 2M at level 2, and 1G at level 3.propis a bitfield tracking the properties of the page ([PageProperty])is_trackedrefers to whether the frame is ref-counted (whentrue) or raw MMIO (whenfalse).
Fields
mapped_pa: usizesize: usizeprop: PagePropertyis_tracked: boolAuto Trait Implementations
impl Freeze for FrameEntryOwner
impl RefUnwindSafe for FrameEntryOwner
impl Send for FrameEntryOwner
impl Sync for FrameEntryOwner
impl Unpin for FrameEntryOwner
impl UnsafeUnpin for FrameEntryOwner
impl UnwindSafe for FrameEntryOwner