pub struct FrameEntryState {
pub mapped_pa: usize,
pub prop: PageProperty,
}Expand description
Verification Design
The proof-side snapshot for a page table leaf: a frame mapped in a node. Asterinas supports huge pages, so it is not necessarily at level 1.
mapped_pais the physical address of the mapped frame.propis a bitfield tracking the properties of the page ([PageProperty])
Fields
mapped_pa: usizeprop: PagePropertyAuto Trait Implementations
impl Freeze for FrameEntryState
impl RefUnwindSafe for FrameEntryState
impl Send for FrameEntryState
impl Sync for FrameEntryState
impl Unpin for FrameEntryState
impl UnsafeUnpin for FrameEntryState
impl UnwindSafe for FrameEntryState